"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deferableSyncApiMethods = [
    'api.log.info',
    'api.log.debug',
    'api.log.warn',
    'api.log.error',
    'api.log.fatal',
    'logConsole'
];
class NotBootstrappedError extends Error {
    constructor(method) {
        super(`You cannot call ${NotBootstrappedError.MAPPED_METHODS_NAMES[method]} methods in the snapshot context`);
    }
}
NotBootstrappedError.MAPPED_METHODS_NAMES = {
    setTimeout: 'timer',
    clearTimeout: 'timer',
    setInterval: 'timer',
    clearInterval: 'timer',
    logConsole: 'logging',
    'api.log.info': 'logging',
    'api.log.debug': 'logging',
    'api.log.warn': 'logging',
    'api.log.error': 'logging',
    'api.log.fatal': 'logging',
    getUserVars: 'process.env',
    getUserPlatform: 'process.platform',
    getUserVersion: 'process.version',
    getUserVersions: 'process.versions',
    'api.getAuthTokenReferences': 'api',
    'api.fetch': 'api',
    'api.requestConfluence': 'api',
    'api.asApp.requestConfluence': 'api',
    'api.asUser.requestConfluence': 'api',
    'api.requestJira': 'api',
    'api.asApp.requestJira': 'api',
    'api.asUser.requestJira': 'api'
};
class PausedBridge {
    constructor() {
        this.deferedCalls = [];
    }
    isDeferableMethod(method) {
        return deferableSyncApiMethods.includes(method);
    }
    getDeferedInvocations() {
        const defered = [...this.deferedCalls];
        this.deferedCalls = [];
        return defered;
    }
    call(fn, ...args) {
        if (!this.isDeferableMethod(fn)) {
            throw new NotBootstrappedError(fn);
        }
        this.deferedCalls.push({
            method: fn,
            args
        });
    }
    callIgnored(fn, ...args) {
        throw new NotBootstrappedError(fn);
    }
    callSync(fn, ...args) {
        throw new NotBootstrappedError(fn);
    }
}
exports.PausedBridge = PausedBridge;
