"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function apiFetch(bridge, bridgeMethodToCall, url, init) {
    return new Promise((resolve, reject) => {
        const resolveWrapper = (response) => {
            const fetchResponse = {
                text: async () => response.data,
                json: async () => JSON.parse(response.data),
                arrayBuffer: async () => response.arrayBufferData,
                ok: response.ok,
                status: response.status,
                statusText: response.statusText
            };
            resolve(fetchResponse);
        };
        const rejectWrapper = (response) => {
            if (response.error)
                reject(response.error);
            reject(response);
        };
        bridge.callIgnored(bridgeMethodToCall, url, init, resolveWrapper, rejectWrapper);
    });
}
exports.apiFetch = apiFetch;
function handleAPIErrors(errorCode, action) {
    switch (errorCode) {
        case 400:
        case 413:
            throw new Error('Bad request');
        case 401:
        case 403:
        case 404:
            throw new Error('Permissions error or key does not exist');
        case 409:
            throw new Error('Conflicting update occurred');
        case 500:
            throw new Error('Internal server error');
        default:
            throw new Error(`Unknown error. Received code ${errorCode} (${action})`);
    }
}
exports.handleAPIErrors = handleAPIErrors;
var Product;
(function (Product) {
    Product[Product["jira"] = 0] = "jira";
    Product[Product["confluence"] = 1] = "confluence";
})(Product = exports.Product || (exports.Product = {}));
