"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTransferableThroughBridge = (obj) => {
    switch (obj.constructor) {
        case String:
        case Number:
        case Boolean:
        case ArrayBuffer:
        case Uint8Array:
        case Int8Array:
        case Uint8ClampedArray:
        case Int16Array:
        case Uint16Array:
        case Int32Array:
        case Uint32Array:
        case Float32Array:
        case Float64Array:
        case DataView:
        case Map:
        case Set:
        case RegExp:
        case Date:
        case Object:
        case Array:
            return true;
        default:
            return false;
    }
};
exports.transformNonTransferablesForLogging = (externalCopy, object) => {
    const visitedObjects = [];
    const isObject = (obj) => obj && typeof obj === 'object';
    const isPromise = (obj) => obj.constructor === Promise;
    const isObjectLiteral = (obj) => obj.constructor === Object;
    const isNullPrototypeObject = (obj) => Object.getPrototypeOf(obj) === null;
    const getTag = (obj) => {
        const desc = Object.getOwnPropertyDescriptor(obj, Symbol.toStringTag);
        if (desc && desc.enumerable === false) {
            return `[${desc.value}]`;
        }
        return '';
    };
    const serializeToExternalCopy = (value, prefix) => {
        return externalCopy({
            value,
            prefix
        });
    };
    function _transform(value) {
        const formattedObject = (constructor, tag, obj) => {
            visitedObjects.push(obj);
            const enumerableProps = _transform({ ...obj });
            visitedObjects.pop();
            const val = [constructor, tag].filter((x) => x !== '').join(' ');
            return serializeToExternalCopy(enumerableProps, val);
        };
        if (typeof value === 'function') {
            return serializeToExternalCopy(`[Function${value.name ? ': ' + value.name : ''}]`, undefined);
        }
        if (typeof value === 'symbol') {
            return serializeToExternalCopy(value.toString(), undefined);
        }
        if (isObject(value)) {
            const tag = getTag(value);
            if (visitedObjects.indexOf(value) > -1) {
                return serializeToExternalCopy('[Circular]', undefined);
            }
            else if (Array.isArray(value)) {
                value = value.map(_transform);
            }
            else if (isPromise(value)) {
                return serializeToExternalCopy('[Promise]', undefined);
            }
            else if (isNullPrototypeObject(value)) {
                return formattedObject('[Object: null prototype]', tag, value);
            }
            else if (isObjectLiteral(value)) {
                if (tag) {
                    return formattedObject('Object', tag, value);
                }
                visitedObjects.push(value);
                const newValue = {};
                Object.entries(value).forEach(([key, val]) => (newValue[key] = _transform(val)));
                Object.getOwnPropertySymbols(value).forEach((symbolKey) => (newValue[`[${symbolKey.toString()}]`] = _transform(value[symbolKey])));
                value = newValue;
                visitedObjects.pop();
            }
            else if (!exports.isTransferableThroughBridge(value)) {
                const name = value.constructor ? value.constructor.name : '';
                return formattedObject(name, tag, value);
            }
        }
        return value;
    }
    return _transform(object);
};
